Class: Rack::Rewritten::Url

Inherits:
Object
  • Object
show all
Defined in:
lib/rack/url.rb

Instance Method Summary collapse

Constructor Details

#initialize(app, &block) ⇒ Url

Returns a new instance of Url.



9
10
11
12
13
14
15
# File 'lib/rack/url.rb', line 9

def initialize(app, &block)
  @app = app
  @translate_backwards = false
  @downcase_before_lookup = false
  @translate_partial = false
  instance_eval(&block) if block_given?
end

Instance Method Details

#call(env) ⇒ Object



17
18
19
20
21
22
23
24
25
26
27
28
29
30
31
32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
58
59
# File 'lib/rack/url.rb', line 17

def call(env)
  req = Rack::Request.new(env)

  subdomain = env["SUBDOMAIN"] ? "#{env["SUBDOMAIN"]}:" : ""

  path = "#{subdomain}#{req.path_info}"
  path.downcase! if downcase_before_lookup?

  if ::Rewritten.includes?(path.chomp("/")) or backwards=( translate_backwards? && ::Rewritten.exist_translation_for?(path) )

    to = ::Rewritten.includes?(path.chomp("/")) || path

    current_path = ::Rewritten.get_current_translation(to)
    current_path = current_path.split(":").last
    current_path = current_path.split('?')[0]

    if (current_path == req.path_info) or (::Rewritten.base_from(req.path_info) == current_path) or ::Rewritten.has_flag?(path, 'L')
      # if this is the current path, rewrite path and parameters
      tpath, tparams = split_to_path_params(to)

      env['QUERY_STRING'] = Rack::Utils.build_query(tparams.merge(req.params))
      req.path_info = tpath + ::Rewritten.appendix(req.path_info)
      @app.call(req.env)
    else
      # if this is not the current path, redirect to current path
      # NOTE: assuming redirection is always to non-subdomain-path

      r = Rack::Response.new

      new_path = env["rack.url_scheme"].dup
      new_path << "://"
      new_path << env["HTTP_HOST"].dup.sub(/^#{subdomain.chomp(':')}\./, '')
      new_path << current_path
      new_path << ::Rewritten.appendix(path) unless backwards
      new_path << '?' << env["QUERY_STRING"] unless (env["QUERY_STRING"]||'').empty?

      r.redirect(new_path, 301)
      a = r.finish
    end
  else
    @app.call(req.env)
  end
end

#split_to_path_params(path_and_query) ⇒ Object



61
62
63
64
# File 'lib/rack/url.rb', line 61

def split_to_path_params(path_and_query)
  path, query_string = path_and_query.split('?').push('')[0..1]
  [path, Rack::Utils.parse_query(query_string)] 
end